@@ -26,9 +26,15 @@
 
 G_BEGIN_DECLS
 
+GtkWidget *go_selector_new_color	(GOColor initial_color,
+					 GOColor default_color,
+					 char const *color_group);
+#ifndef GOFFICE_DISABLE_DEPRECATED
+GOFFICE_DEPRECATED_FOR(go_selector_new_color)
 GtkWidget *go_color_selector_new	(GOColor initial_color,
 					 GOColor default_color,
 					 char const *color_group);
+#endif
 GOColor    go_color_selector_get_color 	(GOSelector *selector, gboolean *is_auto);
 gboolean   go_color_selector_set_color  (GOSelector *selector, GOColor color);
 void       go_color_selector_set_allow_alpha   (GOSelector *selector, gboolean allow_alpha);
